#!python
# The above makes the script executable.

import clize
import os
import sys
import shutil

base_slide = """ 
\\documentclass[aspectratio=43]{beamer}
\\usepackage{etoolbox}
\\newtoggle{overlabel_includesvgs}
\\newtoggle{overlabel_includelabels}

\\toggletrue{overlabel_includesvgs}
\\toggletrue{overlabel_includelabels}

\\input{beamer_slider_preamble.tex}

\\begin{document}
\\begin{frame}
\\maketitle
\\end{frame}

\\begin{frame}\osvg{myoverlay}
\\title{Slide with overlay}
This is some example text!
\\end{frame}

\end{document}
"""

def slider_init(latexfile=None):
    # return
    print("Initializing da slides.")
    wdir = os.getcwd()
    print(wdir)
    if latexfile == None:
        latexfile = "index.tex"
    if not latexfile.endswith(".tex"):
        latexfile += ".tex"
    latexfile = os.path.join(wdir, latexfile)
    if os.path.exists(latexfile):
        print("File already exists", latexfile)
        # sys.exit()
    # Done with the introductory bullshit.

    if not os.path.isdir(os.path.dirname(latexfile)):
        os.makedirs(os.path.dirname(latexfile))

    import jinja2
    with open(latexfile, 'w') as f:
        f.write(base_slide)

    print("Initializing with", latexfile)

    # jinja2.Environment().from_string(base_slide)
    from slider.slide import set_svg_background_images
    set_svg_background_images(latexfile)

    # Initialise a new slider project
    pass

if __name__ == "__main__":
    slider_init("../../test/index.tex")
    # from slider.latexutils import latexmk
    import slider

    # clize.run(slider_init)



from jinjafy import execute_command
import os
import shutil

def latexmk(texfile,pdf_out=None,shell=True,cleanup=False, Linux=False):
    cdir = os.getcwd()
    dname = os.path.dirname(texfile)
    os.chdir(dname)
    texfile = os.path.basename(texfile)
    if Linux:
        CMD = "latexmk -f -g -pdf -interaction=nonstopmode " + texfile
        print("Running LaTeX command>> " + CMD)
        s = execute_command(CMD.split(" "), shell=shell)
    else:
        CMD = "latexmk -f -g -pdf -shell-escape -interaction=nonstopmode " + texfile
        print("Running LaTeX command>> " + CMD)
        s = execute_command(CMD.split(" "),shell=shell)

    if pdf_out:
        shutil.copyfile(texfile[:-4]+".pdf", pdf_out)
    else:
        pdf_out = os.path.join(os.path.dirname(texfile), texfile[:-4]+".pdf")

    if cleanup and os.path.exists(pdf_out):
        bft = ['bbl', 'blg', 'fdb_latexmk', 'fls', 'aux', 'synctex.gz', 'log']
        for ex in bft:
            import glob
            fl = glob.glob(dname + "/*."+ex)
            for f in fl:
                os.remove(f)

    os.chdir(cdir)
    return pdf_out






from jinjafy.cache.simplecache import cache_update_str as cache_update_str
from jinjafy.cache.simplecache import cache_contains_str as cache_contains_str

from jinjafy.cache.simplecache import cache_update_file as cache_update_file
from jinjafy.cache.simplecache import cache_contains_file as cache_contains_file

from jinjafy.cache.simplecache import cache_update_dir as cache_update_dir
from jinjafy.cache.simplecache import cache_contains_dir as cache_contains_dir


from hashlib import md5
import os
import pickle
import glob

def dir_content_cache_(dir, pattern="*"):
    fl = glob.glob(dir + "/" + pattern)
    s = ''.join(fl)
    key = "key_"+dir
    return fl, s,key

def cache_contains_dir(cache_base, dir, pattern="*"):
    # fl = glob.glob(dir)
    fl,s,key =  dir_content_cache_(dir, pattern=pattern)

    v = [cache_contains_file(cache_base, f) for f in fl]
    if all(v) and cache_contains_str(cache_base, key, s):
        return True
    return False

def cache_update_dir(cache_base, dir, pattern="*"):
    fl, s, key = dir_content_cache_(dir, pattern=pattern)
    cache_update_str(cache_base, key, s)
    for f in fl:
        cache_update_file(cache_base, f)


def cache_contains_str(cache_base,key=None,value=None):
    assert(key or value)
    value = hash_binary_(value.encode())
    if not key: key = value
    return cache_contains_hash(cache_base, key, value)

def cache_update_str(cache_base,key,value):
    assert(key or value)
    value = hash_binary_(value.encode())
    if not key: key = value
    return cache_update_hash(cache_base, key, value)


def cache_contains_file(cache_base,file):
    key = os.path.abspath(file)
    if not os.path.exists(file):
        return False
    value = hash_file_(file)
    return cache_contains_hash(cache_base, key, value)

def hash_file_(file):
    import hashlib
    hasher = hashlib.md5()
    with open(file, 'rb') as afile:
        buf = afile.read()
        hasher.update(buf)
    return hasher.hexdigest()

def cache_update_file(cache_base, file):
    key = os.path.abspath(file)
    value = hash_file_(file)
    return cache_update_hash(cache_base, key, value)


def cache_contains_hash(cache_base,key,hash_val):
    cc = load_cache(cache_base)
    return cc.get(key,"Not found") == hash_val

def cache_update_hash(cache_base,key,hash_val):
    cc = load_cache(cache_base)
    cc[key] = hash_val
    save_cache(cache_base, cc)


def hash_binary_(str_bin):
    return md5(str_bin).hexdigest()


def cache_file(cache_base):
    return os.path.join(cache_base, "cache.pkl")

def save_cache(cache_base, cache):
    with open(cache_file(cache_base), 'wb') as f:
        pickle.dump(cache,f)

def load_cache(cache_base):
    if not os.path.exists(cache_file(cache_base)):
        save_cache(cache_base, {'default' : 42})
        return load_cache(cache_base)
    with open(cache_file(cache_base), 'rb') as f:
        return pickle.load(f)


if __name__ == "__main__":
    cache_base = "./"

    print("Hello World")



#!python
# The above makes the script executable.

import slider.legacy_importer
import PyPDF2
import os
from jinjafy import execute_command
# from slider import slide
from slider import legacy_importer
from slider.legacy_importer import SVG_EDIT_RELPATH, SVG_TMP_RELPATH, move_template_files, DTU_beamer_base, svg_edit_to_importable
from jinjafy.cache import cache_update_str, cache_contains_str, cache_contains_file, cache_update_file
import shutil
from slider.slide_fixer import check_svg_file_and_fix_if_broken
from slider.latexutils import latexmk
import clize
import glob

dc = "\\documentclass"

def fix_handout(s):
    i = s.find(dc) + len(dc)
    j1 = s.find('[', i)
    j2 = s.find("{", i)
    if 0 < j1 < j2:
        s = s[:j1 + 1] + "handout," + s[j1 + 1:]
    else:
        s = s[:j2 + 1] + "[handout]" + s[j2 + 1:]
    return s

def set_svg_background_images(lecture_tex, verbose=False,
                              fix_broken_osvg_files=False,
                              recompile_on_change=True,
                              clean_temporary_files=False,
                              copy_template_resource_files=True,
                              force_recompile=False,
                              force_fix_broken_osvg_files = None,
                              ):
    '''
    Main file for fixing/setting osvg background images in the given lecture .pdf.
    Usage:

    > slider <text-file-to-convert>

    :param lecture_tex: File to set background image in.
    :return:
    '''
    MAIN_TEX_DIR = os.path.dirname(lecture_tex)
    SVG_TMP_DIR = MAIN_TEX_DIR + "/" + SVG_EDIT_RELPATH + "/" + SVG_TMP_RELPATH
    SVG_OSVG_DIR = MAIN_TEX_DIR + "/" + SVG_EDIT_RELPATH
    force_fix_broken_osvg_files = [] if force_fix_broken_osvg_files is None else force_fix_broken_osvg_files

    print("Slider is setting the background images for the .tex. file\n>  %s" % os.path.abspath(lecture_tex))
    if copy_template_resource_files:
        move_template_files(output_dir=MAIN_TEX_DIR, output_tex_file=None)
    if not os.path.exists(lecture_tex):
        # move a basic .tex file to this location and proceed
        shutil.copyfile(DTU_beamer_base +"/dtu_slideshow_base.tex", lecture_tex)

    ANY_CHANGES = True
    tex = recursive_tex_apply(lecture_tex)
    tex = "\n".join([tex[k] for k in tex])
    all_tex = tex
    tex = tex.splitlines()

    ol = "\\osvg"
    tex = [s.strip() for s in tex if ol in s and "@ifnextchar" not in s and "%" not in s[:s.find(ol)]] # exclude definition of osvg command
    sinfo = {}

    for s in tex:
        i = s.find(ol) + len(ol)
        if s[i] == "[": i = s.find("]", i)
        i = s.find("{", i)
        ie = s.find("}", i)
        if ie == -1: continue
        s = s[i+1:ie]
        ii = all_tex.find(s)
        frame_start = all_tex.rfind("\\begin{frame}", 0, ii)
        frame_end = all_tex.find("\\end{frame}", ii, len(all_tex))

        cs = all_tex[frame_start:frame_end]
        d = {"pdf_label": s, "svg_edit_file": MAIN_TEX_DIR + "/" + SVG_EDIT_RELPATH + "/" + s + ".svg", 'slide_tex': cs}
        sinfo[s] = d
        # print(d)

    if not os.path.exists(MAIN_TEX_DIR + "/" + SVG_EDIT_RELPATH):
        os.mkdir(MAIN_TEX_DIR + "/" + SVG_EDIT_RELPATH)
    # Prepare alternative .tex file; compile with handout and watermarks for later reference.
    lecture_tex_nosvg = lecture_tex[:-4] + "_NO_SVGS.tex"
    with open(lecture_tex, "r") as f:
        s = f.read()

    if s.find(dc) < 0:
        # find and fix the import
        dc2 = "\\input{"
        j1 = s.find(dc2)+len(dc2)
        j2 = s.find(dc2) + s.find("}", s.find(dc2))
        fhead = MAIN_TEX_DIR + "/" + s[j1:j2]+".tex"
        with open(fhead, 'r') as f:
            sh = f.read()
            sh = fix_handout(sh)
            with open(fhead, 'w') as f2:
                f2.write(sh)
    else:
        s = fix_handout(s)

    i = s.find("\\begin{document}")
    ii = s.rfind("\n", i - 10, i)
    s = s[:ii] + "\n \\togglefalse{overlabel_includesvgs}\n\\toggletrue{overlabel_includelabels}\n" + s[ii:]

    with open(lecture_tex_nosvg, "w") as f:
        f.write(s)

    lecture_tex_nosvg_pdf = lecture_tex_nosvg[:-4] + ".pdf"

    # lecture_tex_nosvg_tex = recursive_tex_apply(lecture_tex_nosvg)

    if cache_contains_str(MAIN_TEX_DIR, key='all_tex', value=all_tex) and os.path.exists(lecture_tex_nosvg_pdf):
        print("slider> Cache contains nosvg tex file")
    else:
        cdir = os.getcwd()
        os.chdir(os.path.dirname(lecture_tex_nosvg))
        execute_command(("latexmk -shell-escape -f -pdf -interaction=nonstopmode " + os.path.basename(lecture_tex_nosvg)).split(" "))
        os.chdir(cdir)
        cache_update_file(MAIN_TEX_DIR, lecture_tex_nosvg)
        ANY_CHANGES = True

    # Make .png background images.
    with open(lecture_tex_nosvg_pdf, 'rb') as f:
        pdfdoc = PyPDF2.PdfFileReader(f)
        for i in range(pdfdoc.getNumPages()):
            content = pdfdoc.getPage(i).extractText()
            for osvg_name, d in sinfo.items(): #enumerate(sinfo):

                if d['pdf_label'] in content:
                    d['pdf_page'] = i
                    d['png_bgimg'] = SVG_TMP_DIR + "/" + d['pdf_label'] + ".png"
                    if not os.path.exists(d['svg_edit_file']):
                        '''
                        Found \osvg{myslide}, but myslide.svg does not exist. Re-create it from the original slide.
                        '''
                        print("Failed to find editable file: %s. Re-creating from snapshot..."%d['svg_edit_file'])
                        tmp_svg_file = "%s/%s/%s"%(os.path.dirname(d['svg_edit_file']),
                                                   SVG_TMP_RELPATH,
                                                   os.path.basename(d['svg_edit_file']))
                        tmp_svg_file = legacy_importer.slide_to_image(lecture_tex_nosvg_pdf, tmp_svg_file, i + 1)
                        legacy_importer.raw_svg_to_osvg(tmp_svg_file, overwrite_existing=True)
                        ANY_CHANGES = True

                    if cache_contains_str(MAIN_TEX_DIR, key=d['pdf_label'], value=d['slide_tex']):
                        # print("slider> Cache contains slide tex; continuing: " + d['pdf_label'] )
                        continue
                    legacy_importer.slide_to_image(lecture_tex_nosvg_pdf, d['png_bgimg'], i + 1)
                    ANY_CHANGES = True
                    cache_update_str(MAIN_TEX_DIR, key=d['pdf_label'], value=d['slide_tex'])


    # This is the step that actually fixes the svg files. i.e. squeeze fonts, etc.
    for osvg_name, d in sinfo.items():
        if (osvg_name+".svg") not in force_fix_broken_osvg_files:
            if cache_contains_file(MAIN_TEX_DIR, d['svg_edit_file']) and not force_recompile:
                continue
        '''        
        Check if the svg image pass sanity checks: Does it exist and is it okay?                
        '''
        if fix_broken_osvg_files:
            check_svg_file_and_fix_if_broken(d['svg_edit_file'], verbose=verbose)

        legacy_importer.svg_edit_to_importable(d['svg_edit_file'], verbose=verbose)
        # legacy_importer.svg_check_background_layer(d['svg_edit_file'], verbose=verbose) # This was an old check for BG img.
        cache_update_file(MAIN_TEX_DIR, d['svg_edit_file'])
        ANY_CHANGES = True

    if ANY_CHANGES and recompile_on_change:

        latexmk(lecture_tex)

    if clean_temporary_files:
        if verbose:
            print("> SlideConverter: Removing temporary dirs...")
        # raise Exception()
        DNE = SVG_OSVG_DIR + "/do_not_edit"
        if os.path.exists(SVG_TMP_DIR):
            for v in glob.glob(SVG_TMP_DIR + "/*"):
                if not v.endswith("png"):
                    os.remove(v)

        if os.path.exists(DNE):
            for v in glob.glob(DNE + "/*"):
                if not v.endswith("png"):
                    os.remove(v)

def slide_no_by_text(pdf_file, text):
    assert False
    # Make .png background images.
    if os.path.exists(pdf_file):
        with open(pdf_file, 'rb') as f:
            print(pdf_file)
            pdfdoc = PyPDF2.PdfFileReader(f)
            for i in range(pdfdoc.getNumPages()):
                content = pdfdoc.getPage(i).extractText()
                # for j, d in enumerate(sinfo):
                if text in content:
                    return i+1
    else:
        print("Warning: slide.py() -> slide_no_by_text(): PDF file not found " + pdf_file)
    return -1
    # raise Exception()

def recursive_tex_apply(doc, fun=None, current_output=None):
    if not fun:
        def mfun(curdoc, txt, cur_out):
            if not cur_out: cur_out = dict()
            cur_out[curdoc] = txt
            return cur_out

        fun = mfun
    if os.path.exists(doc):
        def rfile(doc, encoding):
            with open(doc, 'r', encoding=encoding) as f:
                tex = f.read()
            return tex
        try:
            tex = rfile(doc, encoding="utf-8")
        except Exception as e:
            print("Problem reading file", doc)
            print(e)
            import glob
            from chardet.universaldetector import UniversalDetector
            detector = UniversalDetector()
            detector.reset()
            with open(doc, 'rb') as f:
                detector.feed(f.read())
            detector.close()
            res = detector.result['encoding']
            print("Detecting encoding with chardet...")
            print(res)
            tex = rfile(doc, encoding=detector.result['encoding'])

        current_output = fun(doc, tex, current_output)
        for s in tex.splitlines():
            fs = '\\input{'
            if fs in s and not s.strip().startswith("%"):
                j = s.find(fs)
                rec_file = s[j + len(fs):s.find("}", j)]
                if os.path.isabs(rec_file):
                    rec_file_tex = rec_file
                else:
                    rec_file_tex = os.path.dirname(doc) + "/" + rec_file
                    if not rec_file_tex.endswith(".tex"):
                        rec_file_tex += ".tex"

                current_output = recursive_tex_apply(rec_file_tex, fun, current_output)
    return current_output

def recursive_tex_collect(doc):
    assert False
    sdict = recursive_tex_apply(doc)
    def gathersub(file):
        lines = []
        if file not in sdict:
            print(sdict)
            raise Exception("Bad error occured in split lines " + file )
        for s in sdict[file].splitlines():
            fs = '\\input{'
            if fs in s and not s.strip().startswith("%"):
                j = s.find(fs)
                rec_file = s[j + len(fs):s.find("}", j)]
                if os.path.isabs(rec_file):
                    rec_file_tex = rec_file
                else:
                    rec_file_tex = os.path.dirname(file) + "/" + rec_file
                    if not rec_file_tex.endswith(".tex"):
                        rec_file_tex += ".tex"

                lines += gathersub(rec_file_tex)
            else:
                lines.append(s)
        return lines

    lines = gathersub(doc)
    return "\n".join(lines)




import numpy as np
from fractions import Fraction
import jinja2


def format_list_symbols(list, pattern, symbol="x", seperator=",\ "):
    return format_join(list, pattern=symbol+"_{%i}", seperator=seperator)


def n2w(i):
    w = {0: 'zero', 1: 'one', 2: 'two', 3: 'three', 4: 'four', 5: 'five', 6: 'six', 7: 'seven', 8: 'eight',
         9: 'nine', 10: 'ten'}
    return i if i < 0 or i > 10 else w[i]


def format_list(list_, pattern):
    list_ = tolist(list_)
    return [pattern % s for s in list_]


def format_join(list, pattern, seperator=",\ ",withand=False,withor=False,lastsep=None):
    ls = format_list(list, pattern)
    if withand:
        return seperator.join(ls[:-1]) + "$, and $" + ls[-1]
    if withor:
        return seperator.join(ls[:-1]) + "$, or $" + ls[-1]
    return seperator.join(ls)


def format_join_enum(list, pattern="x_{%i}=%g", seperator=",\ "):
    list = tolist(list)
    return seperator.join(format_list( zip( range(1,len(list)+1 ), list), pattern))


def as_set(l, symbol='f_{%i}'):
    if type(l) != list and type(l) != np.ndarray:
        l = [l]
    l = tolist(l)
    s = [symbol%(i,) for i in l]
    s = '\{' + ", ".join(s) + "\}"
    return s


def as_set_list(ll, symbol='%g'):
    s = []
    for l in ll.flat:
        l = tolist(l)
        s.append(as_set(l, symbol))
    s = ["$"+ds+"$" for ds in s]
    s = ", ".join(s)
    return s


def infty(n,tol=10^8):
    if n > tol:
        s = '\infty'
    else:
        s = str(n)
    return s


def flatten(ar):
    v = []
    if type(ar) is np.ndarray or type(ar) is np.array:
        for x in ar.flat:
            m = flatten(x)
            if type(m) == list:
                v = v + m
            else:
                v.append(m)
    else:
        v = ar
    return v


def tolist(l):
    if type(l) == np.ndarray:
        l2 = []
        for x in l.flat:
            l2.append(  x.tolist() if isinstance(x,np.ndarray) else x  )
        l = l2

    elif type(l) == list or hasattr(l, '__iter__'):
        pass
    else:
        l = [l]
    return l


def jget(A,n=0):
    A = flatten(A)
    return A[n]


def as_rational(x, output='tex', always_frac=False):
    if type(x) == jinja2.runtime.Undefined:
        return "UNDEFINED(jinja2)"
    b = Fraction.from_float(x).limit_denominator(10000)
    s = "output_error_in_as_rational_filter"
    if output == 'tex':
        if (b.denominator == 1 or b.numerator == 0) and not always_frac:
            s = '%i'%b.numerator
        else:
            s = "\\frac{%i}{%i}"%(b.numerator, b.denominator)
    return s


def mylen(l):
    if isinstance(l, np.ndarray):
        sz = l.size
    else:
        sz = len(l)
    return sz


def permute_exam_answers(section,permutation):
    v = section.split("\\item")
    v = v[:5] + v[-1:]
    assert(len(v) == 6)
    permutation = [0] + permutation + [5]
    v[0] = "\\begin{answer}[%i]\n"%permutation.index(1)
    v2 = "\\item".join( [v[i] for i in permutation] )
    return v2


def startswithvowel(value):
    if value.lower().startswith(("a", "e", "i", "o","u")):
        return True
    else:
        return False


def aan(s):
    if s.startswith("no "):
        return ""
    return "an" if startswithvowel(s) else "a"



# Use this guide:
# Use:  pipreqs.exe slider --no-pin --force for requirements.txt
# https://packaging.python.org/tutorials/packaging-projects/
# py -m build && twine upload dist/*

import setuptools
import pkg_resources
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="beamer-slider",
    version="0.1.3",
    author="Tue Herlau",
    author_email="tuhe@dtu.dk",
    description="Software to create inkscape overlays in Beamer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url='https://lab.compute.dtu.dk/tuhe/slider',
    project_urls={
        "Bug Tracker": "https://lab.compute.dtu.dk/tuhe/slider/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
    install_requires=['Jinja2', 'numpy', 'chardet', 'scipy', 'seaborn', 'lxml', 'matplotlib', 'pylatexenc', 'beautifulsoup4', 'PyPDF2'],
    include_package_data=True,
    package_data={'': ['DTU_Beamer_files/*.*']},
    scripts=['scripts/slider_init.py'],
)



"""
Fix broken issues in osvg files.
for instance, you copy a random file, and it has the wrong dimensions, etc.
Need some fairly serious checking I guess...

Example of a good file:

inkscape:groupmode="layer"
     id="layer1"
     inkscape:label="bg_layer"
     style="display:inline"
     sodipodi:insensitive="true">
"""


good_file = """<?xml version="1.0" encoding="UTF-8"?>
<svg xmlns:sodipodi = "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
 xmlns:inkscape = "http://www.inkscape.org/namespaces/inkscape"
 height="297.638pt" version="1.2" viewBox="0 0 396.85 297.638" width="396.85pt" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><metadata></metadata>
<g inkscape:groupmode="layer" id="layer1" inkscape:label="bg_layer" style="display:inline" sodipodi:insensitive="true">
<image
xlink:href="{0}"
width="100%"
height="100%"
preserveAspectRatio="none"
style="image-rendering:optimizeQuality"
id="image4444th"
x="0"
y="0" />
</g>
<g inkscape:groupmode="layer"
id="layer2"
inkscape:label="Layer 1"
style="display:inline">
</svg>
"""

""""
First idea would be to simply load files, replace the tmp-stuff, and check if they agree.
"""
from bs4 import BeautifulSoup
import os

def check_svg_file_and_fix_if_broken(osvg_file, verbose=True):
    assert False
    '''
    Sanity check the given file. Does the slide appears to be in okay shape? Is it broken?
    if it is, fix it.
    '''
    # print(osvg_file)
    png_ = "tmp/" + os.path.basename(osvg_file)[:-4] + ".png"

    ID = 'image4444th'
    to_save = None
    with open(osvg_file, 'r', encoding="UTF-8",errors="surrogateescape") as f:
        soup = BeautifulSoup(f, 'xml', from_encoding="UTF-8")

        bg_tags = soup.findAll("image", {'id': ID})

        if len(bg_tags) == 0:
            print("uh oh. No background image found in", osvg_file)

        g = None
        is_file_ok = True
        for i in soup.findAll("g", {'inkscape:groupmode': 'layer'}):
            if i['inkscape:label'] == "bg_layer":
                g = i
        if g is not None:
            bgim = g.find("image")
            if bgim['id'] == ID:
                # We have a BG image, it has the right ID. Also check if the path matches.
                bg_png = bgim['xlink:href']

                if bg_png != png_:
                    print("Mismatching background PNGs", osvg_file)
                    print(bg_png, png_)
                    bgim['xlink:href'] = png_
                    to_save = soup.prettify(formatter="xml")

            else:
                print("We found the bg_layer tag, but it has no image in it. SVG is broken", osvg_file)
                is_file_ok = False
        else:
            is_file_ok = False

        if is_file_ok:
            # Do sanity check of svg height property.
            height = soup.find('svg')['height']
            if height.find("pt") > 0:
                height = height[:-2]
            height = float(height)
            # float(soup.find('svg')['height'])
            if abs(height - 297.638) > 5:
                is_file_ok = False

    if to_save is not None:
        # raise Exception("asdfsdaf", osvg_file)
        with open(osvg_file, 'w', encoding="UTF-8", errors="surrogateescape") as f:
            f.write(to_save)
        return

    if not is_file_ok:
        # raise Exception("Broken file", osvg_file)
        # File is not ok. We have to fix it. But how?

        gsoup = BeautifulSoup(good_file, 'xml', from_encoding="UTF-8")
        bstag = BeautifulSoup(str(gsoup.svg.g).format(png_), 'lxml', from_encoding="UTF-8")
        g_bg = str(bstag.g)
        with open(osvg_file, 'r', encoding="UTF-8", errors="surrogateescape") as f:
            s = f.read()
            soup = BeautifulSoup(s, 'xml', from_encoding="UTF-8")
            print("Finding all tags")
            # c = 0
            for j in soup.svg.find_all(recursive=False):
                IL= "inkscape:label"
                if IL in j.attrs and j[IL] == "bg_layer":
                    # gsoup = BeautifulSoup(good_file, 'xml', from_encoding="UTF-8")
                    j.replaceWith(bstag.g)
                    # print( j['inkscape:label']  )
                    break
                    # print("FOUND")
                # print(j)
                # c += 1
                # print(c, "-----")
                # if c == 4:
                #     break

            soup.svg.attrs = gsoup.svg.attrs
            s = str(soup)
            # j = s.find("<defs")
            # print("Found defs at j", j)
            # s = good_file.format( png_) + "\n" + s[j:]
        # import time
        # time.sleep(0.1)
        # print(s[:4000])
        # soup2 = BeautifulSoup(s, 'xml', from_encoding="UTF-8")

        with open(osvg_file, 'w', encoding="UTF-8", errors="surrogateescape") as f:
            f.write(s)
    a = 234


from jinjafy import jinjafy_comment
import numpy as np

#"<jinja1>"
#\begin{tabular}{ {{cc}} }
# {% if bookstabs %}\toprule{% endif %}
# {% if vvlabels %}
#   {% for vl in vvlabels %}
#       {% if loop.index > 1 %} & {% endif %}  \multicolumn{ {{vl[0]}} }{ {{vl[2]}} }{ {{vl[1]}} }
#   {% endfor %} \\
#   {% for vl in vvlabels %}
#       {% if vl[3] %}
# 	     \cmidrule(r){ {{vl[3]}} }
#       {% endif %}
#   {% endfor %}
# {% endif %}
# {% for row in X %}
# {% if bookstabs and loop.index == 2%}\midrule{% endif %}
# {% for c in row %}
# {% if loop.index > 1 %} & {% endif %} {{ c['tex'] }} {% if loop.index == W %} \\ {% endif %}
# {% endfor %}
# {% endfor %}
# {% if bookstabs %}\bottomrule{% endif %}
#\end{tabular}
#</jinja1>
# Convert a matrix to a table super quickly
def mat2table(X,vlabels=None,hlabels=None,file_out = None, bookstabs=True, vvlabels=None,plot=False,pdf_out=None, standalone=False):
    X, Xx, Xerr,Xdl = fmat_X2dict(X)
    if pdf_out: plot = True
    #%%
    if plot:
        import matplotlib.pyplot as plt
        #plt.style.use('ggplot')
        plt.style.use('seaborn')
        fig = plt.figure()
        ax = fig.gca()
        #ax = plt.gca()
        ls = []
        for j in range(X.shape[0]):
            ls.append(ax.plot(Xx[j, :]).pop() )

            if Xerr[j]:
                plt.errorbar(range(X.shape[1]), Xx[j,:], yerr=Xerr[j], color=ls[j].get_color())

            for i in range( X.shape[1] ):
                if 'xs' in X[j,i]:
                    plt.plot([i]*len(X[j,i]['xs']), X[j,i]['xs'], '.', color=ls[j].get_color())

        if vlabels:
            plt.legend(ls, vlabels, bbox_to_anchor=(1.04, 1), loc="upper left")
        if hlabels:
            plt.xticks(range(X.shape[1]), hlabels[1:])
        #plt.subplots_adjust(right=0.5)
        plt.tight_layout(rect=[0, 0, 1, 1])
        plt.show()
        #if pdf_out:
        #    fig.savefig(pdf_out, bbox_inches='tight')


    if vlabels:
        vltex =  [{'tex': v} for v in vlabels]
        for i in range(len(Xdl)):
            Xdl[i] = [vltex[i]] + Xdl[i]

    if hlabels:
        Xdl = [ [{'tex': h} for h in hlabels] ] + Xdl

    if vvlabels:
        cc = 1
        for i in range(len(vvlabels)):
            if len(vvlabels[i]) < 3:
                vvlabels[i].append("c")
            dl = vvlabels[i][0]
            if dl == 1:
                a = None
            else:
                a = "%i-%i"%(cc, cc+dl-1)
            cc = cc + dl
            vvlabels[i] = vvlabels[i] + [a]

    H = len(Xdl)
    W = len(Xdl[0])
    cc = ["c" for i in range(W)]
    if vlabels:
        cc[0] = "l"
    cc = "".join(cc)

    def fmat(x):
        if isinstance(x, int):
            x = str(x)
        if isinstance(x, float):
            x = "%2.3f"%x
        return x

    #X = [ [fmat(x) for x in row] for row in X]

    data = {'X' : Xdl, 'hlabels': hlabels, 'vlabels': vlabels, 'cc': cc, 'H':H, 'W': W, 'bookstabs': bookstabs,
            'vvlabels': vvlabels}

    from jinjafy.jinjafy import jinjafy_comment
    s = jinjafy_comment(data,jinja_tag="jinja1")
    if file_out:
        print("Writing to: " + file_out)

        if standalone:
            s = jinjafy_comment({"s": s}, jinja_tag="jinja3")

        with open(file_out, 'w') as f:
            f.write(s)
        if standalone:

            from slider import latexmk
            latexmk(file_out)


    return s
# "<jinja3>"
# \documentclass[crop]{standalone}
# \usepackage{booktabs}
# \usepackage{siunitx}
# \begin{document}
# {{s}}
# \end{document}
# </jinja3>

def fmat_X2dict(X):
    X = np.asarray(X, dtype=np.object)
    if len(X.shape) > 2:
        X2 = np.ndarray(X.shape[:2], dtype=np.object)
        for i in range(X.shape[0]):
            for j in range(X.shape[1]):
                X2[i, j] = X[i, j, :].squeeze()
        X = X2
    X = np.reshape(X, X.shape[:2])

    for i in range(X.shape[0]):
        for j in range(X.shape[1]):
            dx = X[i,j]
            if isinstance(dx, (list, np.ndarray)):
                dx = [x for x in np.ravel(dx)]

            if not isinstance(dx, dict):
                dx = {'x': dx}
            elif not isinstance(dx['x'], str):
                x = dx['x']
                # if isinstance(x, np.ndarray):
                if 'tex' not in dx:
                    dx['std'] = np.std(x)
                    dx['std_mean'] = np.std(x) / np.sqrt( len(x))
                    dx['xs'] = x
                    dx['x'] = np.mean(x)
                    x2, u2 = mround( dx['x'], dx['std_mean'] )

                    dx['tex'] = '\\SI{%g\\pm %.2f}{}'%(x2, u2)

            if 'tex' not in dx:
                dx['tex'] = dx['x']

            X[i,j] = dx

    Xerr = [None] * X.shape[0]
    Xx = np.zeros(X.shape)

    for i in range(X.shape[0]):
        if "std" in X[0,0]:
            Xerr[i] = [dx['std_mean'] for dx in X[i]]

        for j in range(X.shape[1]):
            Xx[i,j] = X[i,j]['x']

    Xdl = []
    for i in range(X.shape[0]):
        dx = []
        for j in range(X.shape[1]):
            dx.append(X[i,j])
        Xdl.append(dx)


    return X,Xx,Xerr,Xdl

import math
def mround(x,u):
    n = np.floor(np.log10(x)+1)
    dx = np.round(x / np.power(10.0, n), 2)
    du = np.round(u / np.power(10.0, n), 2)
    return dx * np.power(10, n), du * np.power(10.0,n)

